from imblearn.ensemble import BalancedBaggingClassifier
import numpy as np


from ..monoview_classifiers.random_scm import ScmBagging
from ..monoview.monoview_utils import BaseMonoviewClassifier
from ..utils.base import base_boosting_estimators
from ..utils.hyper_parameter_search import CustomRandint, CustomUniform

classifier_class_name = "IBRSCM"
proto=True

class IBRSCM(BaseMonoviewClassifier, BalancedBaggingClassifier):

    def __init__(self, random_state=None, n_estimators=10,
                 sampling_strategy="auto", replacement=False,
                 base_estimator=ScmBagging(), **kwargs):
        super(IBRSCM, self).__init__(random_state=random_state,
                                               base_estimator=base_estimator,
                                         n_estimators=n_estimators,
                                         sampling_strategy=sampling_strategy,
                                         replacement=replacement)

        self.param_names = ["n_estimators", "sampling_strategy",
                            "base_estimator__n_estimators",
                            "base_estimator__max_rules",
                            "base_estimator__max_samples",
                            "base_estimator__max_features",
                            "base_estimator__model_type",
                            "base_estimator__p_options",]
        self.classed_params = ["base_estimator"]
        self.distribs = [CustomRandint(low=1, high=50),
                         ["auto"],CustomRandint(low=1, high=300),
                         CustomRandint(low=1, high=20),
                         CustomUniform(), CustomUniform(),
                         ["conjunction", "disjunction"], CustomUniform(), ]
        self.weird_strings=[]

    def fit(self, X, y):
        print(self.base_estimator.n_estimators)
        BalancedBaggingClassifier.fit(self, X, y)
        self.feature_importances_ = np.zeros(X.shape[1])
        for estim in self.estimators_:
            if hasattr(estim['classifier'], 'feature_importances_'):
                self.feature_importances_ += estim['classifier'].feature_importances_
        self.feature_importances_ /= np.sum(self.feature_importances_)
        print('Fitted')
        return self



