from imblearn.ensemble import BalancedBaggingClassifier
import numpy as np
from SamBA.samba import NeighborHoodClassifier, ExpTrainWeighting
from SamBA.relevances import *
from SamBA.distances import *
from sklearn.preprocessing import RobustScaler



from ..monoview.monoview_utils import BaseMonoviewClassifier
from ..utils.base import base_boosting_estimators
from ..utils.hyper_parameter_search import CustomRandint, CustomUniform

classifier_class_name = "IBSamba"

class IBSamba(BaseMonoviewClassifier, BalancedBaggingClassifier):

    def __init__(self, random_state=None, n_estimators=10,
                 sampling_strategy="auto", base_estimator=NeighborHoodClassifier(),
                 replacement=False, **kwargs):
        super(IBSamba, self).__init__(random_state=random_state,
                                         base_estimator=base_estimator,
                                         n_estimators=n_estimators,
                                         sampling_strategy=sampling_strategy,
                                         replacement=replacement)

        self.param_names = ["n_estimators", "sampling_strategy",
                            "base_estimator__n_estimators",
                            "base_estimator__relevance",
                            "base_estimator__distance",
                            "base_estimator__difficulty", "base_estimator__b", "base_estimator__pred_train", "base_estimator__normalizer",
                            "base_estimator__normalize_dists", "base_estimator__a", "base_estimator__class_weight",]
        self.distribs = [CustomRandint(low=1, high=50),
                         ["auto"],CustomRandint(low=1, high=70),
                         [ExpRelevance()],
                         [EuclidianDist(), PolarDist(), ExpEuclidianDist(), Jaccard()],
                         [ExpTrainWeighting()],
                         CustomUniform(0.1, 6,),
                         [True, False],
                         [RobustScaler()],
                         [True], CustomRandint(0, 10, 'e-'),
                         ["balanced", None],]
        self.weird_strings ={}

    def fit(self, X, y):
        BalancedBaggingClassifier.fit(self, X, y)
        self.feature_importances_ = np.zeros(X.shape[1])
        for estim in self.estimators_:
            if hasattr(estim['classifier'], 'feature_importances_'):
                self.feature_importances_ += estim['classifier'].feature_importances_
        self.feature_importances_ /= np.sum(self.feature_importances_)
        return self



