from SamBA.samba import NeighborHoodClassifier, ExpTrainWeighting
import numpy as np
from sklearn.tree import DecisionTreeClassifier
from SamBA.relevances import *
from SamBA.distances import *
from sklearn.preprocessing import RobustScaler

from ..monoview.monoview_utils import BaseMonoviewClassifier
from ..utils.hyper_parameter_search import CustomRandint, CustomUniform


# Author-Info
__author__ = "Baptiste Bauvin"
__status__ = "Prototype"  # Production, Development, Prototype


# class Decis
classifier_class_name = "SamBAClf"

class SamBAClf(NeighborHoodClassifier, BaseMonoviewClassifier):

    def __init__(self, base_estimator=DecisionTreeClassifier(max_depth=1,
                                                       splitter='best',
                                                       criterion='gini'),
                 n_estimators=2,
                 estimator_params=tuple(),
                 relevance=ExpRelevance(),
                 distance=EuclidianDist(),
                 difficulty=ExpTrainWeighting(),
                 keep_selected_features=True,
                 normalizer=RobustScaler(),
                 b=2, a=0.01,
                 pred_train=False,
                 forced_diversity=False,
                 normalize_dists=False,
                 class_weight="balanced",
                 **kwargs):
        """

        Parameters
        ----------
        random_state
        model_type
        max_rules
        p
        kwargs
        """
        super(SamBAClf, self).__init__(base_estimator=base_estimator,
                 n_estimators=n_estimators,
                 estimator_params=estimator_params,
                 relevance=relevance,
                 distance=distance,
                 difficulty=difficulty,
                 keep_selected_features=keep_selected_features,
                 normalizer=normalizer,
                 forced_diversity=forced_diversity,
                 b=b, a=a, pred_train=pred_train,
                                       normalize_dists=normalize_dists,
                                       class_weight=class_weight)
        self.param_names = ["n_estimators",
                            "relevance",
                            "distance",
                            "difficulty", "b", "pred_train", "normalizer",
                            "normalize_dists", "a", "class_weight",]
        self.distribs = [CustomRandint(low=1, high=70),
                         [ExpRelevance()],
                         [EuclidianDist(), PolarDist(), ExpEuclidianDist(), Jaccard()],
                         [ExpTrainWeighting()],
                         CustomUniform(0.1, 6,),
                         [True, False],
                         [RobustScaler()],
                         [True], CustomRandint(0, 10, 'e-'),
                         ["balanced", None],
                         ]
        self.classed_params = []
        self.weird_strings = {}

    def get_interpretation(self, directory, base_file_name, y_test, feature_ids,
                           multi_class=False):
        interpret_string = self.get_feature_importance(directory, base_file_name, feature_ids)
        return interpret_string
