import numpy as np
from sklearn.preprocessing import LabelBinarizer

from spkm.spkm_wrapper import SPKMlikeSklearn
from spkm.kernels_and_gradients import RBFKernel, PolyKernel

from ..monoview.monoview_utils import BaseMonoviewClassifier
from ..utils.hyper_parameter_search import CustomRandint
from ..utils.dataset import get_samples_views_indices

classifier_class_name = "SPKM"
proto=True

class SPKM(BaseMonoviewClassifier, SPKMlikeSklearn):

    def __init__(self, random_state=42, n_u=2, kernel=RBFKernel(0.5),
                 spkmregP=1, spkminit="randn",
                 nspkminits=10, preprocessinglist=[0,1,2], **kwargs):

        SPKMlikeSklearn.__init__(self, random_state=random_state,
                                         n_u=n_u,
                                         kernel=kernel,
                                         spkmregP=spkmregP,
                                         spkminit=spkminit,
                                         nspkminits=nspkminits,
                                         preprocessinglist=preprocessinglist)
        self.param_names = ["n_u", "kernel", "spkmregP",
                            "spkminit", "nspkminits", "preprocessinglist",
                            "random_state"]
        self.distribs = [[2], [PolyKernel({"d":3, "r":1}), RBFKernel(0.5)],
                         CustomRandint(-4,4, multiplier='e'),
                         ["data"],  [10],
                         [[0,1],],
                         [random_state],]
        self.more_than_two_views = False
        self.weird_strings = []
        self.random_state = random_state

    def fit(self, X, y):
        self.lb =  LabelBinarizer(pos_label=1, neg_label=-1)
        y = self.lb.fit_transform(y)
        return SPKMlikeSklearn.fit(self, X, y[:,0],)

    def predict(self, X, preprocess=True):
        return self.lb.inverse_transform(np.sign(SPKMlikeSklearn.predict(self, X)))

    def get_interpretation(self, directory, base_file_name, labels, multiclass=False):
        u = self.feature_interpretability()
        importances_sum = np.sum(u)
        self.feature_importances_ = u/importances_sum
        return ""

    def accepts_multi_class(self, random_state, n_samples=10, dim=2,
                            n_classes=3, n_views=2):
        return False
